// modelSwarm.java

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;
import swarm.defobj.SymbolImpl;
import swarm.defobj.FArguments;
import swarm.defobj.FArgumentsImpl;
import swarm.defobj.FCall;
import swarm.defobj.FCallImpl;
import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.FActionForEach;
import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.collections.List;
import swarm.collections.ListImpl;
import swarm.simtoolsgui.ProbeDisplayImpl;
import swarm.objectbase.EmptyProbeMapImpl;

public class ModelSwarm extends SwarmImpl
	{
	// global variables for the class
	public Schedule modelSchedule;
	public ActionGroup modelActions;
	public int modelTime;
	public int maxTime;
	public int startBudget;
	public int maxBudget;
	public int notFinished;
	public int numberOfConsumers;
	public ListImpl listOfConsumers;


// constructor for modelSwarm
	public ModelSwarm (Zone aZone)
		{
		super(aZone);
		EmptyProbeMapImpl modelProbeMap=new EmptyProbeMapImpl(getZone(),getClass());
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("maxTime",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("numberOfConsumers",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("startBudget",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("maxBudget",getClass()));
		Globals.env.probeLibrary.setProbeMap$For(modelProbeMap, getClass());
		}

// build the consumer object
	public Object buildObjects()
		{
		int i;
		int name;

		super.buildObjects();
		
// create the list of consumers
		listOfConsumers=new ListImpl(getZone());

// iterate over all possible consumers
		for (i=1;i<=numberOfConsumers;++i)
			{
			Consumer aConsumer;
// name of consumer=index i
			name=i;
// create the consumers
			aConsumer=new Consumer(getZone(),name,maxBudget,startBudget);
// create the mapOfSpending and the arrayOfVisits
			aConsumer.createMapOfSpending(getZone());
			aConsumer.createArrayOfVisits(getZone(),maxTime+1);
// add consumer to the list
			listOfConsumers.addFirst(aConsumer);
			}
		return this;
		}

// define the marketDay actions
	public Object marketDay()
		{
		int go;
		int spending;
		int budget;
		swarm.collections.ListIndex i=null;
		Consumer listElement;

// iterate over the list of consumers
// first create the index
		i=listOfConsumers.listBegin(getZone());
		while ((listElement=(Consumer)i.next())!=null)
			{
// update the budget of the consumer
			budget=listElement.findBudget();

// is he going to the market?
			go=listElement.goToTheMarket();

			if (go==1)
				{
				spending=listElement.spend();

// add 1 to array of visits, at position corresponding to current
// modelTime
				listElement.updateVisits(modelTime,1);
// add spending at the key modelTime in the mapOfSpending
				listElement.updateSpending(modelTime,spending);
// now print a report of the consumer's actions
				System.out.println("This is time "+modelTime);
				System.out.println("I am consumer " + listElement.getConsumerName());
				System.out.println("My current budget is "+ listElement.getBudget());
				System.out.println("Did I go to the market? (from array) "+ listElement.getVisitValue(modelTime));
				System.out.println("I spent (from map) "+ listElement.getSpendingValue(modelTime));
				System.out.println("I have " + listElement.calculateRemainingBudget() + " of currency left.");
				}
			else
				{
// add 0 to array of visits, at position corresponding to current
// modelTime
				listElement.updateVisits(modelTime,0);
// add 0 to map of spending, at position corresponding to current
// modelTime
				listElement.updateSpending(modelTime,0);
// now print a report of the consumer's actions
				System.out.println("This is time "+modelTime);
				System.out.println("I am consumer " + listElement.getConsumerName());
				System.out.println("My current budget is "+ listElement.getBudget());
				System.out.println("Did I go to the market? (from array) "+ listElement.getVisitValue(modelTime));
				System.out.println("I have " + listElement.getBudget() + " of currency left.");
				}
			}// end of iteration of list of consumers
// it is good practice to drop unused objects like indexes when they
// are no longer needed
			i.drop();
		return this;
		}

// at the end of each period, modelTime needs to increase by 1
	public void increaseTime()
		{
		++modelTime;
		}

// the program should stop if it has run long enough
// in this case, if modelTime>maxTime
// return 0 or 1 for observer
	public int checkToStop()
		{
		if (modelTime<=maxTime)
			{
			notFinished=1;
			return 0;
			}
		else
			{
			notFinished=0;
			return 1;
			}
		}
		
// build actions
	public Object buildActions()
		{
// create the action group for all actions to be performed at each time
// (it is trivial, and not necessary here, as there is only one action:
// marketDay)
		modelActions=new ActionGroupImpl(getZone());
		try
			{
			modelActions.createActionTo$message(this,
				new Selector(getClass(),"marketDay",false));
			modelActions.createActionTo$message(this,
				new Selector(getClass(),"increaseTime",false));
			modelActions.createActionTo$message(this,
				new Selector(getClass(),"checkToStop",false));
			}
		catch (Exception e)
			{
			e.printStackTrace(System.err);
			System.exit(1);
			}

// now schedule the actions in time
		modelSchedule=new ScheduleImpl(getZone(),2);
		modelSchedule.at$createAction(0,modelActions);
		return this;
		}

// activity
	public Activity activateIn(Swarm swarmContext)
		{
		super.activateIn(swarmContext);
		modelSchedule.activateIn(this);
		return this.getActivity();
		}

// methods to pass objects to other classes
	public ListImpl getListOfConsumers()
		{
		return (ListImpl)listOfConsumers;
		}

	public Consumer getConsumer(int name)
		{
// note that elements are entered using the addFirst methods
// so the first element is the latest agent created
		return (Consumer)listOfConsumers.atOffset(listOfConsumers.getCount()-name);
		}

	public int getCurrentTime()
		{
		return modelTime;
		}
	}// end of class modelSwarm