// modelSwarm.java

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;
import swarm.defobj.SymbolImpl;
import swarm.defobj.FArguments;
import swarm.defobj.FArgumentsImpl;
import swarm.defobj.FCall;
import swarm.defobj.FCallImpl;
import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.FActionForEach;
import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.collections.List;
import swarm.collections.ListImpl;
import swarm.simtoolsgui.ProbeDisplayImpl;
import swarm.objectbase.EmptyProbeMapImpl;
import swarm.space.Object2dDisplayImpl;
import swarm.space.Grid2dImpl;
import swarm.space.Discrete2dImpl;

public class ModelSwarm extends SwarmImpl
	{
	// global variables for the class
	public Schedule modelSchedule;
	public ActionGroup modelActions;
	public int modelTime;
	public int maxTime;
	public int startBudget;
	public int maxBudget;
	public int notFinished;
	public int numberOfConsumers;
	public ListImpl listOfConsumers;
	public int worldWidth;
	public int worldHeight;
	public int sizeOfMarket;
	public int xMin;
	public int xMax;
	public int yMin;
	public int yMax;
	public Grid2dImpl world;
	public Discrete2dImpl market;

// constructor for modelSwarm
	public ModelSwarm (Zone aZone)
		{
		super(aZone);
		EmptyProbeMapImpl modelProbeMap=new EmptyProbeMapImpl(getZone(),getClass());
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("maxTime",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("numberOfConsumers",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("startBudget",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("maxBudget",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("worldWidth",getClass()));
		modelProbeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("worldHeight",getClass()));
		Globals.env.probeLibrary.setProbeMap$For(modelProbeMap, getClass());
		}

// build the model objects
	public Object buildObjects()
		{
		int i;
		int x;
		int y;
		int name;
		EmptyProbeMapImpl consumerProbe;
		super.buildObjects();
// set position of market in world (+/- centre)
		xMin=(worldWidth-sizeOfMarket)/2;
		xMax=xMin+sizeOfMarket;
		yMin=(worldHeight-sizeOfMarket)/2;
		yMax=yMin+sizeOfMarket;
// initialise the world as a grid2d and fill with null objects
		world=new Grid2dImpl(getZone(),worldWidth,worldHeight);
		world.fillWithObject(null);
// initialise market as a discrete2d and fill the relevant part in yellow
		market=new Discrete2dImpl(getZone(),worldWidth,worldHeight);
		for (y=yMin;y<=yMax;++y)
			{
			for (x=xMin;x<=xMax;++x)
				{
				market.putValue$atX$Y(2,x,y);
				}
			}
// create the list of consumers
		listOfConsumers=new ListImpl(getZone());

// iterate over all possible consumers
		for (i=1;i<=numberOfConsumers;++i)
			{
			Consumer aConsumer;
// name of consumer=index i
			name=i;
// create the consumers
			aConsumer=new Consumer(getZone(),name,maxBudget,startBudget,0);
// set position to negative values
			aConsumer.setPositionX$Y(-999,-999);
// create the mapOfSpending and the arrayOfVisits
			aConsumer.createMapOfSpending(getZone());
			aConsumer.createArrayOfVisits(getZone(),maxTime+1);
// position in world
			this.findPositionInWorld$For$ExcludeMarket(world,aConsumer,1);
// probe for consumer
			consumerProbe=new EmptyProbeMapImpl(getZone(),aConsumer.getClass());
			consumerProbe.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("myName",aConsumer.getClass()));
			Globals.env.probeLibrary.setProbeMap$For(consumerProbe,aConsumer.getClass());
// add consumer to the list
			listOfConsumers.addFirst(aConsumer);
			}
		return this;
		}
		
// find position in world: puts the consumer in the world, excluding
// the market space if the last argument is 1, or only in the market
// space if the last argument is 0.
// First, it checks for the actual coordinates of the consumer
// object: if they are not negative (meaning that the consumer
// already is on the space), they start by putting a null object
// at this position. Then it looks for a random position for the
// consumer object, which has positive coordinates, is in the required
// part of the space (inside or outside market), and in an empty spot.
	public Object findPositionInWorld$For$ExcludeMarket(Grid2dImpl aWorld,Consumer aGuy,int exclude)
		{
		int trialX=-999;
		int trialY=-999;
// put null if consumers are already in the space
		if ((aGuy.getPositionX()>=0)&&(aGuy.getPositionY()>=0))
			{
			aWorld.putObject$atX$Y(null,aGuy.getPositionX(),aGuy.getPositionY());
			}
// put consumers randomly in the space
// first, if exclude is 1, outside the market part
		if (exclude==1)
			{
			while ( ((trialX<0)&&(trialY<0)) ||
				((trialX>=xMin)&&(trialX<=yMin)&&(trialY>=yMin)&&(trialY<=yMax)) ||
				(world.getObjectAtX$Y(trialX,trialY)!=null) )
				{
				trialX=Globals.env.uniformIntRand.getIntegerWithMin$withMax(0,worldWidth-1);
				trialY=Globals.env.uniformIntRand.getIntegerWithMin$withMax(0,worldHeight-1);
				}
			}
// then, if exclude is 0, inside the market part
		else if (exclude==0)
			{
			while ( ((trialX<0)&&(trialY<0)) ||
				(world.getObjectAtX$Y(trialX,trialY)!=null) )
				{
				trialX=Globals.env.uniformIntRand.getIntegerWithMin$withMax(xMin,xMax);
				trialY=Globals.env.uniformIntRand.getIntegerWithMin$withMax(yMin,yMax);
				}
			}
		else
			{
			System.out.println("Wrong value for exclude");
			System.exit(1);
			}
		aGuy.setPositionX$Y(trialX,trialY);
		world.putObject$atX$Y(aGuy,trialX,trialY);
		return this;
		}

// define the marketDay actions
	public Object marketDay()
		{
		int go;
		int spending;
		int budget;
		swarm.collections.ListIndex i=null;
		Consumer listElement;

// iterate over the list of consumers
// first create the index
		i=listOfConsumers.listBegin(getZone());
		while ((listElement=(Consumer)i.next())!=null)
			{
// update the budget of the consumer
			budget=listElement.findBudget();

// is he going to the market?
			go=listElement.goToTheMarket();

			if (go==1)
				{
				spending=listElement.spend();

// add 1 to array of visits, at position corresponding to current
// modelTime
				listElement.updateVisits(modelTime,1);
// add spending at the key modelTime in the mapOfSpending
				listElement.updateSpending(modelTime,spending);
// change position in space to inside the market (this will also
// change the colour of the consumer
				this.findPositionInWorld$For$ExcludeMarket(world,listElement,0);
// now print a report of the consumer's actions
				System.out.println("This is time "+modelTime);
				System.out.println("I am consumer " + listElement.getConsumerName());
				System.out.println("My current budget is "+ listElement.getBudget());
				System.out.println("Did I go to the market? (from array) "+ listElement.getVisitValue(modelTime));
				System.out.println("I spent (from map) "+ listElement.getSpendingValue(modelTime));
				System.out.println("I have " + listElement.calculateRemainingBudget() + " of currency left.");
				}
			else
				{
// add 0 to array of visits, at position corresponding to current
// modelTime
				listElement.updateVisits(modelTime,0);
// add 0 to map of spending, at position corresponding to current
// modelTime
				listElement.updateSpending(modelTime,0);
// change position in space to outside the market
				this.findPositionInWorld$For$ExcludeMarket(world,listElement,1);
// now print a report of the consumer's actions
				System.out.println("This is time "+modelTime);
				System.out.println("I am consumer " + listElement.getConsumerName());
				System.out.println("My current budget is "+ listElement.getBudget());
				System.out.println("Did I go to the market? (from array) "+ listElement.getVisitValue(modelTime));
				System.out.println("I have " + listElement.getBudget() + " of currency left.");
				}
			}// end of iteration of list of consumers
// it is good practice to drop unused objects like indexes when they
// are no longer needed
			i.drop();
		return this;
		}

// at the end of each period, modelTime needs to increase by 1
	public void increaseTime()
		{
		++modelTime;
		}

// the program should stop if it has run long enough
// in this case, if modelTime>maxTime
// return 0 or 1 for observer
	public int checkToStop()
		{
		if (modelTime<=maxTime)
			{
			notFinished=1;
			return 0;
			}
		else
			{
			notFinished=0;
			return 1;
			}
		}
		
// build actions
	public Object buildActions()
		{
// create the action group for all actions to be performed at each time
// (it is trivial, and not necessary here, as there is only one action:
// marketDay)
		modelActions=new ActionGroupImpl(getZone());
		try
			{
			modelActions.createActionTo$message(this,
				new Selector(getClass(),"marketDay",false));
			modelActions.createActionTo$message(this,
				new Selector(getClass(),"increaseTime",false));
			modelActions.createActionTo$message(this,
				new Selector(getClass(),"checkToStop",false));
			}
		catch (Exception e)
			{
			e.printStackTrace(System.err);
			System.exit(1);
			}

// now schedule the actions in time
		modelSchedule=new ScheduleImpl(getZone(),2);
		modelSchedule.at$createAction(0,modelActions);
		return this;
		}

// activity
	public Activity activateIn(Swarm swarmContext)
		{
		super.activateIn(swarmContext);
		modelSchedule.activateIn(this);
		return this.getActivity();
		}

// methods to pass objects to other classes
	public ListImpl getListOfConsumers()
		{
		return (ListImpl)listOfConsumers;
		}

	public Consumer getConsumer(int name)
		{
// note that elements are entered using the addFirst methods
// so the first element is the latest agent created
		return (Consumer)listOfConsumers.atOffset(listOfConsumers.getCount()-name);
		}

	public int getCurrentTime()
		{
		return modelTime;
		}

	public int getWorldWidth()
		{
		return worldWidth;
		}
	public int getWorldHeight()
		{
		return worldHeight;
		}
	public Grid2dImpl getWorld()
		{
		return world;
		}
	public Discrete2dImpl getMarket()
		{
		return market;
		}
	}// end of class modelSwarm