// ObserverSwarm.java
import swarm.Globals; // no # but a ; at the end
import swarm.Selector;
import swarm.defobj.Zone;
import swarm.defobj.ZoneImpl;
import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.objectbase.Swarm;
import swarm.objectbase.VarProbe;
import swarm.objectbase.MessageProbe;
import swarm.objectbase.EmptyProbeMapImpl;
import swarm.gui.Colormap;
import swarm.gui.ColormapImpl;
import swarm.gui.ZoomRaster;
import swarm.gui.ZoomRasterImpl;
import swarm.analysis.EZGraph;
import swarm.analysis.EZGraphImpl;
import swarm.simtoolsgui.GUISwarm;
import swarm.simtoolsgui.GUISwarmImpl;
import swarm.defobj.LispArchiverImpl;
import swarm.space.Object2dDisplay;
import swarm.space.Object2dDisplayImpl;
import swarm.space.Value2dDisplayImpl;
import swarm.gui.ZoomRasterImpl;
import swarm.gui.ZoomRaster;
import swarm.gui.Colormap;
import swarm.gui.ColormapImpl;
import swarm.defobj.Zone;
import swarm.Selector;

public class ObserverSwarm extends GUISwarmImpl
	{
// declare variables
	public int displayFrequency;
	public int displayConsumerName;

	public ActionGroup displayActions;
	public Schedule displaySchedule;

	public ModelSwarm modelSwarm;

	public EZGraphImpl spendingGraph;
	public EZGraphImpl consumerGraph;

	public ColormapImpl colorMap;
	public ZoomRasterImpl worldRaster;
	public Object2dDisplayImpl worldDisplay;
	public Value2dDisplayImpl marketDisplay;
	public int zoomFactor;

// constructor
	public ObserverSwarm(Zone aZone)
		{
		super(aZone);
// create probe map
		EmptyProbeMapImpl probeMap=new EmptyProbeMapImpl(getZone(),getClass());
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("displayFrequency",getClass()));
		probeMap.addProbe(Globals.env.probeLibrary.getProbeForVariable$inClass("displayConsumerName",getClass()));
		Globals.env.probeLibrary.setProbeMap$For(probeMap, getClass());
		}// end of constructor
		
// build Objects
	public Object buildObjects()
		{
		LispArchiverImpl archiver;
		
		super.buildObjects();
// create the archiver object
		archiver=new LispArchiverImpl(getZone(),"parameters.scm");
// import the parameters (note the casting as ModelSwarm). The archiver
// will also use automatically the constructor for the ModelSwarm
		modelSwarm=(ModelSwarm)archiver.getWithZone$key(getZone(),"modelSwarm");
// create probe displays
	    Globals.env.createArchivedProbeDisplay(modelSwarm,"modelSwarm");
    	Globals.env.createArchivedProbeDisplay(this, "observerSwarm");
// set control panel to stop
		getControlPanel().setStateStopped();
// build models objects
		modelSwarm.buildObjects();
// create graphs
// spendingGraph shows the average, total and minimum spending
// of consumers during the game
// note the ease of use of the constructor:
//public EZGraphImpl(Zone aZone,
//                   String aTitle,
//                   String xl,
//                   String yl,
//                   String windowGeometryRecordName)
		spendingGraph=new EZGraphImpl(getZone(),"Agents' spending",
				"Timex2","Spending","spendingGraph");
// create the average,total,min sequences (remember the try/catch blocks
// necessary with selectors)
	    try
			{
      		spendingGraph.createAverageSequence$withFeedFrom$andSelector 
        		("Average spending",modelSwarm.getListOfConsumers(),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
      		spendingGraph.createTotalSequence$withFeedFrom$andSelector 
        		("Total spending",modelSwarm.getListOfConsumers(),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
      		spendingGraph.createMinSequence$withFeedFrom$andSelector 
        		("Minimum spending",modelSwarm.getListOfConsumers(),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
    		}
		catch (Exception e)
			{
      		System.err.println ("Exception in creating spendingGraph: " 
                          + e.getMessage ());
    		} 
// consumerGraph shows the frequentation and spending on the market
// of a consumer (determined by displayConsumerName).
		consumerGraph=new EZGraphImpl(getZone(),"A consumer",
				"Timex2","Visit/Spending","consumerGraph");
// create the sequences (remember the try/catch blocks
// necessary with selectors)
	    try
			{
      		consumerGraph.createSequence$withFeedFrom$andSelector 
        		("Went to the market",modelSwarm.getConsumer(displayConsumerName),
         		new Selector(Class.forName("Consumer"), "getVisit", 
                       false));
      		consumerGraph.createSequence$withFeedFrom$andSelector 
        		("Spent",modelSwarm.getConsumer(displayConsumerName),
         		new Selector(Class.forName("Consumer"), "getSpending", 
                       false));
    		}
		catch (Exception e)
			{
      		System.err.println ("Exception in creating consumerGraph: " 
                          + e.getMessage ());
    		}
		
// create raster objects
		zoomFactor=10;
// colorMap
		colorMap=new ColormapImpl(getZone());
		colorMap.setColor$ToName((byte)0,"black");
		colorMap.setColor$ToName((byte)1,"blue");
		colorMap.setColor$ToName((byte)2,"yellow");
		colorMap.setColor$ToName((byte)3,"red");
// world raster
		worldRaster=new ZoomRasterImpl(getZone());
		worldRaster.setColormap(colorMap);
		worldRaster.setZoomFactor(zoomFactor);
		worldRaster.setWidth$Height(modelSwarm.getWorldWidth(),modelSwarm.getWorldHeight());
		worldRaster.setWindowTitle("A little town");
		worldRaster.pack();
// market display
		marketDisplay=new Value2dDisplayImpl(getZone(),worldRaster,colorMap,modelSwarm.getMarket());
// world display
		try
			{
			worldDisplay=new Object2dDisplayImpl(getZone(),worldRaster,
				modelSwarm.getWorld(),new Selector(Class.forName("Consumer"),"drawSelfOn",false));
			worldRaster.setButton$Client$Message(3,worldDisplay,
				new Selector(worldDisplay.getClass(),"makeProbeAtX$Y",true));
			}
		catch(Exception e)
			{
			e.printStackTrace(System.err);
			System.exit(1);
			}
		worldDisplay.setObjectCollection(modelSwarm.getListOfConsumers());
// display the world raster at the beginning
		worldRaster.erase();
		marketDisplay.display();
		worldDisplay.display();
		worldRaster.drawSelf();
		
		return this;
		}// end of observer buildObjects

	public Object buildActions()
		{
		super.buildActions();
		modelSwarm.buildActions();
// define displayActions
		displayActions=new ActionGroupImpl(getZone());
		try
			{
			displayActions.createActionTo$message(spendingGraph,
				new Selector(spendingGraph.getClass(),"step",true));
			displayActions.createActionTo$message(consumerGraph,
				new Selector(consumerGraph.getClass(),"step",true));
			displayActions.createActionTo$message(worldRaster,
				new Selector(worldRaster.getClass(),"erase",true));
			displayActions.createActionTo$message(marketDisplay,
				new Selector(marketDisplay.getClass(),"display",true));
			displayActions.createActionTo$message(worldDisplay,
				new Selector(worldDisplay.getClass(),"display",true));
			displayActions.createActionTo$message(worldRaster,
				new Selector(worldRaster.getClass(),"drawSelf",true));
			displayActions.createActionTo$message(this,
				new Selector(getClass(),"observerCheckToStop",false));
			displayActions.createActionTo$message(Globals.env.probeDisplayManager,
				new Selector(Globals.env.probeDisplayManager.getClass(),"update",true));
			displayActions.createActionTo$message(getActionCache(),
				new Selector(getActionCache().getClass(),"doTkEvents",true));
			}
		catch (Exception e)
			{		  
			e.printStackTrace(System.err);
			System.exit(1);
			}
System.out.println("create actions done for observer");
// now schedule the actions in time
		displaySchedule=new ScheduleImpl(getZone(),2);
		displaySchedule.at$createAction(1,displayActions);
		return this;
		}// end of observer buildActions

// activity
	public Activity activateIn(Swarm swarmContext)
		{
		super.activateIn(swarmContext);
		modelSwarm.activateIn(this);
		displaySchedule.activateIn(this);
		return this.getActivity();
		}

// observerCheckToStop
	public Object observerCheckToStop()
		{
		if(modelSwarm.checkToStop()==1)
			{
			System.out.println("THE MODEL STOPPED RUNNING");
			getControlPanel().setStateStopped();
			}
		return this;
		}

	}// end of observer class